import jax.numpy as jnp

from .defaults import default_weight_init
from .meta import Parameter, ParameterModel

__all__ = [
  'DecompositionParameter', 'decomposition',
]

class DecompositionParameter(Parameter):
  def __init__(self, shape, n, w1=default_weight_init, w2=default_weight_init, name=None, **properties):
    shape1 = (shape[0], n) + shape[2:]
    shape2 = (n, shape[1]) + shape[2:]

    self.w1 = w1(
      shape=shape1,
      name=(name + '_w1') if name is not None else None
    )

    self.w2 = w2(
      shape=shape2,
      name=(name + '_w2') if name is not None else None
    )

    super(DecompositionParameter, self).__init__(
      self.w1, self.w2,
      shape=shape, name=name,
      **properties
    )

  def get_output_for(self, w1, w2):
    return jnp.tensordot(w1, w2, axes=[(1,), (0,)])

  def get_output_shape_for(self, w1_shape, w2_shape):
    assert w1_shape[1] == w2_shape[0], 'parameters shapes are not compatible'
    assert w1_shape[2:] == w2_shape[2:], 'parameters shapes are not compatible'
    return (w1_shape[0], w2_shape[1]) + w1_shape[2:]

class decomposition(ParameterModel):
  ParameterType = DecompositionParameter

  def __init__(self, n, w1=default_weight_init, w2=default_weight_init, name=None, composite=True, **properties):
    super().__init__({**properties, 'composite': composite}, n, w1=w1, w2=w2, name=name)